@@ -6,7 +6,6 @@ diff -u . . 2>/dev/null && diffArgs="$diffArgs -u"	# Unified diff output
 
 ec=0
 
-set -o pipefail
 set -e
 
 PROCESSING=""
@@ -14,7 +13,7 @@ print_status() {
     echo "Error while processing $PROCESSING"
 }
 
-trap print_status ERR
+trap print_status EXIT
 
 if [ "x${top_srcdir}" = "x" ]; then
   top_srcdir=".."
@@ -41,4 +40,8 @@ for ref in ${top_srcdir}/tests/*.asn1.-*; do
 	fi
 done
 
+if [ $ec = 0 ]; then
+    trap '' EXIT
+fi
+
 exit $ec
